<?php

/**
 * Thrown if a data value failed its validation test.
 */
class InvalidDataException extends Exception { }


/**
 * Thrown if a datum is present for which there is no validator.
 */
class InvalidKeyException extends Exception { }


/**
 * TODO: document this
 */
function validateEmail($v) {

      // TODO: See http://www.linuxjournal.com/article/9585. Note that this
      // complete definition of email addresses may be overkill for your
      // purposes, but also note that addresses like n@ai are valid,
      // routable internet email addresses.

}


/**

 * @param $v A user ID to normalize and validate. A valid user ID is a
 * string containing 1 or more decimal digits.
 *
 * @return A normalized copy of $v.
 *
 * @throws InvalidDataException if $v could not be normalized or validated.
 */
function validateUserID($v) {
      $v = trim($v);
      if (1 != preg_match('/^\d+$/', $v))
            throw new InvalidDataException('validateUserID: ' . $v);
      return $v;
}


/**
 * @param $v A credit card number to normalize and validate. For validation,
 * the mod 10 function is applied, as well as matching the number prefix and
 * the number length for each brand of card.
 *
 * @return A normalized copy of $v.
 *
 * @throws InvalidDataException if $v could not be normalized or validated.
 */
function validateCreditCardNumber($v) {
      $v = ereg_replace('[^0-9]', '', $v);

      $ln = strlen($v);
      if ($ln < 13 || $ln > 16)
            throw new InvalidDataException('validateCreditCardNumber: ' .
                                           $v . ' bad length');

      $prfx1 = substr($v, 0, 1);
      $prfx2 = substr($v, 0, 2);
      $prfx3 = substr($v, 0, 3);
      $prfx4 = substr($v, 0, 4);

      if ($prfx2 >= 51 && $prfx2 <= 55) {
            if (16 != $ln)
                  throw new InvalidDataException('validateCreditCardNumber: ' .
                                                 $v .  ' invalid MasterCard');
      }
      elseif (4 == $prfx1) {
            if (13 != $ln && 16 != $ln)
                  throw new InvalidDataException('validateCreditCardNumber: ' .
                                                 $v .  ' invalid Visa');
      }
      elseif (34 == $prfx2 || 37 == $prfx2) {
            if (15 != $ln)
                  throw new InvalidDataException('validateCreditCardNumber: ' .
                                                 $v .  ' invalid American Express');
      }
      elseif (36 == $prfx2 || 38 == $prfx2 || ($prfx3 >= 300 && $prfx3 <= 305)) {
            if (14 != $ln)
                  throw new InvalidDataException('validateCreditCardNumber: ' .
                                                 $v .  ' invalid Diners Club');
      }
      elseif (6011 == $prfx4) {
            if (16 != $ln)
                  throw new InvalidDataException('validateCreditCardNumber: ' .
                                                 $v .  ' invalid Discover');
      }
      elseif (3 == $prfx1 || 1800 == $prfx4 || 2131 == $prfx4) {
            if (15 != $ln && 16 != $ln)
                  throw new InvalidDataException('validateCreditCardNumber: ' .
                                                 $v .  ' invalid JCB');
      }
      else
            throw new InvalidDataException('validateCreditCardNumber: unknown type '
                                           . $v);

      // Compute mod 10 checksum.
      $rv = strrev($v);
      $chksm = 0;

      for ($i = 0; $i < $ln; $i++) {
            $c = substr($cardNumber, $i, 1);

            if (1 == $i % 2)
                  $c *= 2;

            if ($c > 9) {
                  $x = $c % 10;
                  $y = ($c - $x) / 10;
                  $c = $x + $y;
            }
      
            $chksm += $c;
      }

      if (0 != ($chksm % 10))
            throw new InvalidDataException('validateCreditCardNumber: ' .
                                           $v . ' fails mod 10 checksum.');

      return $v;
}


/**
 * @param $v An iPhone IPID (UDID) to normalize and validate. A valid IPID
 * is a string of 40 hexadecimal digits. This function allows the digits to
 * be upper- or lowercase, but normalizes to lowercase.
 *
 * @return A normalized copy of $v.
 *
 * @throws InvalidDataException if $v could not be normalized or validated.
 */
function validateIpid($v) {
      $v = strtolower(trim($v));

      if (1 != preg_match('/^[\da-z]{40}$/', $v))
            throw new InvalidDataException('validateUserID: ' . $v);

      return $v;
}


/**
 * @param $v The value to compare to the known-good list of values in
 * $whitelist. Before checking against $whitelist, $v will be normalized
 * with trim.
 *
 * @param whitelist The list of known-good values.
 *
 * @return A normalized copy of $v.
 *
 * @throws InvalidDataException if $v could not be normalized or validated.
 */
function validateWhitelist($v, $whitelist) {
      $v = trim($v);

      if (! array_key_exists($v, $whitelist))
            throw new InvalidDataException('validateWhitelist: ' . $v);

      return $v;
}


/**
 * @param $items The items to validate.
 *
 * @param $validators An associative array mapping keys to validation
 * functions ("validators"). For each key in $items, we call its associated
 * validation function with the key's value as the argument to the
 * validator.
 *
 * @param $tolerant Whether or not to ignore keys in $items that lack an
 * associated validator un $validators.
 *
 * @return An associative array of the normalized and validated values,
 * mapped with the right keys. If $tolerant is true, the return value will
 * not contain keys/values for keys in $items without validators.
 *
 * @throws InvalidKeyException If $tolerant is false, any of the keys in
 * $items do not have an associated validator in $validators.
 */
function validate($items, $validators, $tolerant) {
      $r = array();

      foreach ($items as $k => $v) {
            if (! array_key_exists($k, $validators)) {
                  if ($tolerant)
                        continue;
                  throw new InvalidKeyException($k . ': ' . $v);
            }

            $r[$k] = $validators[$k]($v);
      }

      return $r;
}

?>

